Imports System
Imports System.Runtime.InteropServices
Imports System.Collections.Generic

Public Module uCalcNET
#If PLATFORM = "x64" Then
   Const uCalcDLL As String = "uCalc64.dll"
#Else
   Const uCalcDLL As String = "uCalc32.dll"
#End If

   Public Delegate Sub uCalcDelegate(ByVal Expr As IntPtr)

   ''' <summary>S: Data types that come with uCalc</summary>
   ''' <remarks>These are built-in data types that come with uCalc</remarks>
   Public Enum BuiltInTypesEnum As Int32
      Type_Unspecified
      Type_Auto
      ''' <summary> Boolean -- </summary>
      Type_Boolean
      ''' <summary> Int8 +++ Rename these to things like Type_Int8</summary>
      Type_Integer_8
      ''' <summary> Byte</summary>
      Type_Integer_8u
      ''' <summary> Int16</summary>
      Type_Integer_16
      ''' <summary> Int16u</summary>
      Type_Integer_16u
      ''' <summary> Int32</summary>
      Type_Integer_32
      ''' <summary> Int32u</summary>
      Type_Integer_32u
      ''' <summary> Int64</summary>
      Type_Integer_64
      ''' <summary> Int64u</summary>
      Type_Integer_64u
      ''' <summary> Int128</summary>
      Type_Integer_128
      ''' <summary> Int128u</summary>
      Type_Integer_128u
      ''' <summary> Single</summary>
      Type_Float_Single
      ''' <summary> Double</summary>
      Type_Float_Double
      ''' <summary> Extended</summary>
      Type_Float_Extended
      Type_Size_t
      Type_Complex
      Type_Decimal
      Type_BigNum
      Type_Date
      Type_String
      ''' <summary> LPCSTR</summary>
      Type_String_LPCSTR
      ''' <summary> FixedStr</summary>
      Type_String_Fixed
      ''' <summary> WideStr</summary>
      Type_String_Wide
      ''' <summary> StackList</summary>
      Type_List_Stack
      ''' <summary> TableList</summary>
      Type_List_Table
      ''' <summary> SortedList</summary>
      Type_List_Sorted
      Type_Null
      Type_Void
      Type_Any
      Type_Pointer
      Type_Omni
      Type_UserDefined
      Type_DataType
      Type_Namespace
      Type_IntPtr
      Type_ADDR
      Type_uCalcPtr
      Type_Item
      Type_ParsedExpr
      Type_uCalcCallback
      Type_Object
      Type_Intrinsic
   End Enum

   ''' <summary>S: Data type constructors</summary>
   Public Enum DataTypeConstruction As Int32
      ByteSize
      AllocArray
      AllocScalar
      AllocByPtr
      ReleaseArray
      ReleaseScalar
      ResetScalar
      ResetArray
      SetArrayValue
      SetScalarValue
      Swap
   End Enum

   ''' <summary>S: Error hander responses</summary>
   Public Enum ErrorHandlerResponseEnum As Int32
      ''' <summary> Immediately stops parsing</summary>
      ErrorAbort
      ''' <summary> Continues parsing (after the error is resolved)</summary>
      ErrorResume
      ''' <summary> Raises the same error again for the next error handler to take care of</summary>
      ErrorReRaise
   End Enum

   ''' <summary>S: Error numbers that uCalc may return</summary>
   Public Enum ErrorNumberEnum As Int32
      ''' <summary> No error</summary>
      Err_None
      ''' <summary> Dynamically defined user error</summary>
      Err_Dynamically_Defined
      ''' <summary> Syntax error</summary>
      Err_Syntax_Error
      ''' <summary> Undefined identifier</summary>
      Err_Undefined_Identifier
      ''' <summary> File not found</summary>
      Err_FileNotFound
      ''' <summary> Unrecognized token</summary>
      Err_Unrecognized_Token
      ''' <summary> Unrecognized command</summary>
      Err_Unrecognized_Command
      ''' <summary> Data type mismatch</summary>
      Err_Datatype_Mismatch
      ''' <summary> Invalid number of arguments</summary>
      Err_Invalid_Argument_Count
      ''' <summary> Invalid definition</summary>
      Err_Invalid_Definition
      ''' <summary> Code block delimiter error</summary>
      Err_CodeBlock_Error
      ''' <summary> Undefined callback</summary>
      Err_Undefined_Callback
      ''' <summary> Error message already defined</summary>
      Err_ErrorMessageAlreadyDefined
      ''' <summary> This item cannot be modified</summary>
      Err_ItemCannotBeModified
      ''' <summary> Unrecognized Class_ member</summary>
      Err_Unrecognized_Class_Member
      ''' <summary> Unrecognized Namespace_ member</summary>
      Err_Unrecognized_Namespace_Member
      ''' <summary> Unbalanced quote</summary>
      Err_Unbalanced_Quote
      ''' <summary> Value cannot be assigned here</summary>
      Err_ValueCannotBeAssigned
      ''' <summary> Array Or_ list bound exceeded</summary>
      Err_Array_Bounds_Exceeded
      ''' <summary> Division by 0</summary>
      Err_Float_Division_By_Zero
      ''' <summary> Inexact result</summary>
      Err_Float_Inexact_Result
      ''' <summary> Invalid operation</summary>
      Err_Float_Invalid_Operation
      ''' <summary> Floating point overflow</summary>
      Err_Float_Overflow
      ''' <summary> Floating point underflow</summary>
      Err_Float_Underflow
      Err_Customized1 = 100
      Err_Customized2
      Err_Customized3
      Err_Customized4
      Err_Customized5
      Err_Customized6
      Err_Customized7
      Err_Customized8
      Err_Customized9
      Err_Customized10
      Err_Customized11
      Err_Customized12
      Err_Customized13
      Err_Customized14
      Err_Customized15
      Err_Customized16
      Err_Customized17
      Err_Customized18
      Err_Customized19
      Err_Customized20
   End Enum

   ''' <summary>S: Operator_ grouping associativity</summary>
   Public Enum GroupingEnum As Int32
      GroupLeftToRight
      GroupRightToLeft
   End Enum

   ''' <summary>S: Properties that a uCalc item may have</summary>
   ''' <remarks>
   '''Items defined with uCalc typically have a combination of properties.  A variable defined
   '''with DefineVariable for instance will have the ItemIsEnum.Variable property set.  An Operator_
   '''will have the ItemIsEnum.Operator_ property set.  If it's an infix Operator_ in addition to
   '''having the ItemIsEnum.Operator_ property set ItemIsEnum.Infix will also be set.
   '''property will also be set.
   '''
   '''The IsProperty Function_ can determine if a given uCalc.Item has a certain property.
   '''
   '''You can use the ItemOfProperty Function_ to cycle through the list of all items that share
   '''certain properties Or_ to disambiguate between items that share the same name but have
   '''different roperties (for instance the + Operator_ with the Prefix property vs another one
   '''with the Infix property).
   '''
   '''Some members are keywords in VB.NET.  To avoid naming conflict the following members have been
   '''renamed in the VB.NET header: Function_ Operator_ Optional_ ByVal_ ByRef_ Class_ And_ NameSpace_.
   '''</remarks>
   Public Enum ItemIsEnum As Int32
      AnyItem
      LeftToRight
      RightToLeft
      DataType
      Array
      Class_
      Namespace_
      Variable
      LocalVariable
      Function_
      Operator_
      FunctionOrOperator
      Prefix
      Infix
      Postfix
      AddressOwner
      ByVal_
      ByRef_
      ByCopy
      ByExpr
      ByHandle
      IntPtr
      Optional_
      VariableNumberOfArgs
      RegEx
      LiteralItem
      Locked
      Transform
      InTable
      ItemDef
      ErrorHandler
      StopNow
      ApiCall
      CaseSensitive
      QuotedText
      ChangeMulti
      ExpandArg
      IgnoreStatementSep
      IgnoreCodeBlock
      SyntaxStop
      SkipEvalExpand
      ExprUnit
      EagerArg
      SkipSyntax
      WhiteSpaceCounts
      IgnoreQuote
      UpToEndOrSeparator
      UpToUserStop
      EvalStrOutput
      SkipOutput
      DefaultSettings
      NotFound
      MiscItem
   End Enum

   ''' <summary>S: Licensing options for uCalc products</summary>
   Public Enum LicenseTypeEnum As Int32
      License_Demo
      License_FMP_Free
      License_FMP_Standard_32
      License_FMP_Standard_64
      License_FMP_32_and_64
      License_LanguageBuilder
      License_GeneralParser
      License_Interpreter
      License_ConsoleCalc
      License_WindowsCalc
      License_StringLib
      License_Transform
      License_TransformLib
      License_Search
      License_SearchLib
      License_Library1
      License_Library2
      License_Beta
      License_Debugger
      License_uCalc_All
      License_Developer
      License_EndUser
   End Enum

   ''' <summary>S: Types of tokens</summary>
   Public Enum TokenEnum As Int32
      None
      ''' <summary> A token defined without any specified property is generic (Default_)</summary>
      Generic
      ''' <summary> Represents the absence of any character (token not found)</summary>
      Empty
      ''' <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
      AlphaNumeric
      ''' <summary> A token that serves as argument separator (typically a comma)</summary>
      ArgSeparator
      ''' <summary> A token that starts a code block; typically characters like { [ Or_ (</summary>
      CodeBlock
      ''' <summary> The closing token that matches a CodeBlock token such as } ] Or_ )</summary>
      CodeBlockMatch
      ''' <summary> A token that separates an object from its member typically a . (a dot)</summary>
      MemberAccess
      ''' <summary> A token that represents a literal value typically a quoted string</summary>
      Literal
      ''' <summary> A token that represents a statement separator typically a ; (semicolon)</summary>
      StatementSep
      ''' <summary> A token that represents whitespace typically tab Or_ space characters</summary>
      WhiteSpace
      ''' <summary> A token that is not delimited by word boundaries (+ - * etc) consecutive characters are reduced until they match a value that exists in the symbol table Or_ until 1 character</summary>
      Reducible
      ''' <summary> Non-character that represents the end of a stream of text</summary>
      Eof
      CurrentThread 
      LocalVar
      StaticVar
      ''' <summary> Item in parenthesis (Or_ quotes) following this token will be defined while the expression is parsed</summary>
      DefineNow = StaticVar
      TempDef
      ''' <summary> The expression in parathesis (Or_ quotes) following the token is evaluated And_ inserted in place of the token in the string</summary>
      EvalInsert
      EvalDuringParse
      ''' <summary> Works like EvalInsert except the expression is expanded instead of evaluated</summary>
      ExpandInsert
      ''' <summary> The contents of the quoted file name following this token is inserted into the string</summary>
      FileInclude
      CurrentSpace
      DefSpace
      EvalSpace
   End Enum

   ''' <summary>S: Boolean True_ Or_ False_</summary>
   Public Enum TrueFalseEnum As Int32
      ''' <summary> For compilers in which False_ is a keyword use False_ instead</summary>
      False_
      ''' <summary> For compilers in which True_ is a keyword use True_ instead</summary>
      True_
   End Enum

   ''' <summary>S: Floating point environment exceptions</summary>
   Public Enum FPEnvExceptionEnum As Int32
      ''' <summary> Floating point division by zero Or_ operation resulting in infinity occured (pole error)</summary>
      DivisionByZero = 8
      ''' <summary> The result is not an exact floating point number</summary>
      Inexact = 1
      ''' <summary> Argument passed to a Function_ Or_ Operator_ was invalid (domain error)</summary>
      Invalid = 16
      ''' <summary> The result is too large to be represented as a floating point value of the return type (overflow range error)</summary>
      Overflow = 4
      ''' <summary> The result is too small to be represented as a floating point value of the return type causing loss of precision (underflow range error)</summary>
      Underflow = 2
      ''' <summary> Represents selection of all the floating point exceptions: DivisionByZero Inexact Invalid Overflow And_ Underflow (Bitwise ORed)</summary>
      AllExceptions = 31
   End Enum

   ''' <summary>S: Floating point rouding options</summary>
   Public Enum FPEnvRoundingEnum As Int32
      ''' <summary> Rounds downward (towards negative infinity)</summary>
      Round_Down = 512
      ''' <summary> Rounds towards nearest value</summary>
      Round_ToNearest = 0
      ''' <summary> Rounds towards zero</summary>
      Round_TowardZero = 768
      ''' <summary> Rounds upward (towards positive infinity)</summary>
      Round_Up = 256
   End Enum

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Evaluate")> _ 
   Private Function Evaluate__(ByVal ExprHandle As IntPtr) As  Double
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="EvaluateInt32")> _ 
   Private Function EvaluateInt32__(ByVal ExprHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="EvaluateStr")> _ 
   Private Function EvaluateStr__(ByVal ExprHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="EvaluateVoid")> _ 
   Private Function EvaluateVoid__(ByVal ExprHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Execute")> _ 
   Private Sub Execute__(ByVal ExprHandle As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReleaseExpr")> _ 
   Private Sub ReleaseExpr__(ByVal ExprHandle As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Handle_Overload_ParsedExpr")> _ 
   Private Function Handle_Overload_ParsedExpr__(ByVal ExprHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ElementCount")> _ 
   Private Function ElementCount__(ByVal ItemHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DataType")> _ 
   Private Function DataType__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FunctionAddress")> _ 
   Private Function FunctionAddress__(ByVal ItemHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="IsProperty")> _ 
   Private Function IsProperty__(ByVal ItemHandle As IntPtr,ByVal ItemProperty As ItemIsEnum) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Name_Overload_Item")> _ 
   Private Function Name_Overload_Item__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NextOverload")> _ 
   Private Function NextOverload__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Precedence_Overload_Item")> _ 
   Private Function Precedence_Overload_Item__(ByVal ItemHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReleaseItem")> _ 
   Private Sub ReleaseItem__(ByVal ItemHandle As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Rename")> _ 
   Private Sub Rename__(ByVal ItemHandle As IntPtr,ByVal NewName As String)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReplacementText")> _ 
   Private Function ReplacementText__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValue")> _ 
   Private Sub SetVariableValue__(ByVal ItemHandle As IntPtr,ByVal Value As Double)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueAny")> _ 
   Private Sub SetVariableValueAny__(ByVal ItemHandle As IntPtr,ByVal Value As String)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueByte")> _ 
   Private Sub SetVariableValueByte__(ByVal ItemHandle As IntPtr,ByVal Value As Byte)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueInt16")> _ 
   Private Sub SetVariableValueInt16__(ByVal ItemHandle As IntPtr,ByVal Value As Int16)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueInt32")> _ 
   Private Sub SetVariableValueInt32__(ByVal ItemHandle As IntPtr,ByVal Value As Int32)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueInt64")> _ 
   Private Sub SetVariableValueInt64__(ByVal ItemHandle As IntPtr,ByVal Value As Int64)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueSng")> _ 
   Private Sub SetVariableValueSng__(ByVal ItemHandle As IntPtr,ByVal Value As Single)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueDbl")> _ 
   Private Sub SetVariableValueDbl__(ByVal ItemHandle As IntPtr,ByVal Value As Double)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueStr")> _ 
   Private Sub SetVariableValueStr__(ByVal ItemHandle As IntPtr,ByVal Value As String)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValuePtr")> _ 
   Private Sub SetVariableValuePtr__(ByVal ItemHandle As IntPtr,ByVal Value As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetVariableValueByPtr")> _ 
   Private Sub SetVariableValueByPtr__(ByVal ItemHandle As IntPtr,ByVal Value As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Text")> _ 
   Private Function Text__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="uCalc_Overload_Item")> _ 
   Private Function uCalc_Overload_Item__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Value")> _ 
   Private Function Value__(ByVal ItemHandle As IntPtr) As  Double
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueByte")> _ 
   Private Function ValueByte__(ByVal ItemHandle As IntPtr) As  Byte
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueInt16")> _ 
   Private Function ValueInt16__(ByVal ItemHandle As IntPtr) As  Int16
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueInt32")> _ 
   Private Function ValueInt32__(ByVal ItemHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueInt64")> _ 
   Private Function ValueInt64__(ByVal ItemHandle As IntPtr) As  Int64
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueSng")> _ 
   Private Function ValueSng__(ByVal ItemHandle As IntPtr) As  Single
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueDbl")> _ 
   Private Function ValueDbl__(ByVal ItemHandle As IntPtr) As  Double
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueStr")> _ 
   Private Function ValueStr__(ByVal ItemHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValuePtr")> _ 
   Private Function ValuePtr__(ByVal ItemHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueAddr")> _ 
   Private Function ValueAddr__(ByVal ItemHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Handle_Overload_Item")> _ 
   Private Function Handle_Overload_Item__(ByVal ItemHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Arg")> _ 
   Private Function Arg__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As  Double
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgCount")> _ 
   Private Function ArgCount__(ByVal ExprPartHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgObj")> _ 
   Private Function ArgObj__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgAddr")> _ 
   Private Function ArgAddr__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgExpr")> _ 
   Private Function ArgExpr__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgInt32")> _ 
   Private Function ArgInt32__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgInt64")> _ 
   Private Function ArgInt64__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As  Int64
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgPtr")> _ 
   Private Function ArgPtr__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ArgStr")> _ 
   Private Function ArgStr__(ByVal ExprPartHandle As IntPtr,ByVal Index As Int32) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Return")> _ 
   Private Sub Return__(ByVal ExprPartHandle As IntPtr,ByVal Value As Double)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReturnInt32")> _ 
   Private Sub ReturnInt32__(ByVal ExprPartHandle As IntPtr,ByVal Value As Int32)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReturnInt64")> _ 
   Private Sub ReturnInt64__(ByVal ExprPartHandle As IntPtr,ByVal Value As Int64)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReturnPtr")> _ 
   Private Sub ReturnPtr__(ByVal ExprPartHandle As IntPtr,ByVal Value As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReturnStr")> _ 
   Private Sub ReturnStr__(ByVal ExprPartHandle As IntPtr,ByVal Value As String)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorRaise")> _ 
   Private Function ErrorRaise__(ByVal ExprPartHandle As IntPtr,ByVal ErrNumber As ErrorNumberEnum) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorRaiseMessage")> _ 
   Private Function ErrorRaiseMessage__(ByVal ExprPartHandle As IntPtr,ByVal ErrMessage As String) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="uCalc_Overload_Callback")> _ 
   Private Function uCalc_Overload_Callback__(ByVal ExprPartHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Handle_Overload_Callback")> _ 
   Private Function Handle_Overload_Callback__(ByVal ExprPartHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="OpItem")> _ 
   Private Function OpItem__(ByVal ExprPartHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="AllocArray")> _ 
   Private Function AllocArray__(ByVal DataTypeHandle As IntPtr,ByVal ArraySize As Int32) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="AllocByPtr")> _ 
   Private Function AllocByPtr__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="AllocScalar")> _ 
   Private Function AllocScalar__(ByVal DataTypeHandle As IntPtr, ByVal Value As String  ) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ByteSize")> _ 
   Private Function ByteSize__(ByVal DataTypeHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Index")> _ 
   Private Function Index__(ByVal DataTypeHandle As IntPtr) As  BuiltInTypesEnum
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Item_Overload_DataType")> _ 
   Private Function Item_Overload_DataType__(ByVal DataTypeHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Name_Overload_DataType")> _ 
   Private Function Name_Overload_DataType__(ByVal DataTypeHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReleaseArray")> _ 
   Private Sub ReleaseArray__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReleaseScalar")> _ 
   Private Sub ReleaseScalar__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ResetArray")> _ 
   Private Sub ResetArray__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ResetScalar")> _ 
   Private Sub ResetScalar__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetArray")> _ 
   Private Sub SetArray__(ByVal DataTypeHandle As IntPtr,ByVal Destination As IntPtr , ByVal Source As IntPtr , ByVal Index As Int32)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetScalar")> _ 
   Private Sub SetScalar__(ByVal DataTypeHandle As IntPtr,ByVal Destination As IntPtr , ByVal Source As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Swap")> _ 
   Private Sub Swap__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtrA As IntPtr , ByVal ValuePtrB As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ToString_Overload_DataType_ByPtr")> _ 
   Private Function ToString_Overload_DataType_ByPtr__(ByVal DataTypeHandle As IntPtr,ByVal ValuePtr As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ToString_Overload_DataType_ByStr")> _ 
   Private Function ToString_Overload_DataType_ByStr__(ByVal DataTypeHandle As IntPtr,ByVal Value As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Handle_Overload_DataType")> _ 
   Private Function Handle_Overload_DataType__(ByVal DataTypeHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="AddErrorHandler")> _ 
   Private Function AddErrorHandler__(ByVal uCalcHandle As IntPtr,ByVal ErrorHandlerAddress As uCalcDelegate , ByVal Position As Int32  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="CreateuCalc")> _ 
   Private Function CreateuCalc__(ByVal uCalcHandle As IntPtr) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DataTypeOfIndex")> _ 
   Private Function DataTypeOfIndex__(ByVal uCalcHandle As IntPtr,ByVal Index As BuiltInTypesEnum) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Define")> _ 
   Private Function Define__(ByVal uCalcHandle As IntPtr,ByVal Expression As String , ByVal Address As IntPtr  , ByVal ExistingItem As IntPtr  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineConstant")> _ 
   Private Function DefineConstant__(ByVal uCalcHandle As IntPtr,ByVal Definition As String , ByVal VariableAddress As IntPtr  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineDataType")> _ 
   Private Function DefineDataType__(ByVal uCalcHandle As IntPtr,ByVal TypeSize As Int32 , ByVal AllocArrayValue As IntPtr , ByVal AllocScalar As IntPtr , ByVal AllocScalarByPtr As IntPtr , ByVal DeleteArray As IntPtr , ByVal DeleteScalar As IntPtr , ByVal ResetScalar As IntPtr , ByVal ResetArray As IntPtr , ByVal SetValueArray As IntPtr , ByVal SetValueScalar As IntPtr , ByVal SwapValues As IntPtr , ByVal ToString As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineFunction")> _ 
   Private Function DefineFunction__(ByVal uCalcHandle As IntPtr,ByVal Definition As String , ByVal FunctionAddress As uCalcDelegate  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineOperator")> _ 
   Private Function DefineOperator__(ByVal uCalcHandle As IntPtr,ByVal Definition As String , ByVal iPrecedence As Int32 , ByVal Grouping As GroupingEnum  , ByVal FunctionAddress As uCalcDelegate  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineSyntax")> _ 
   Private Function DefineSyntax__(ByVal uCalcHandle As IntPtr,ByVal Definition As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineTransform")> _ 
   Private Function DefineTransform__(ByVal uCalcHandle As IntPtr,ByVal FromExpr As String , ByVal ToExpr As String , ByVal ItemHandle As IntPtr  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="DefineVariable")> _ 
   Private Function DefineVariable__(ByVal uCalcHandle As IntPtr,ByVal Definition As String , ByVal VariableAddress As IntPtr  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorExpression")> _ 
   Private Function ErrorExpression__(ByVal uCalcHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorLocation")> _ 
   Private Function ErrorLocation__(ByVal uCalcHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorMessage")> _ 
   Private Function ErrorMessage__(ByVal uCalcHandle As IntPtr, ByVal Index As ErrorNumberEnum) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorNumber")> _ 
   Private Function ErrorNumber__(ByVal uCalcHandle As IntPtr) As  ErrorNumberEnum
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorResponse")> _ 
   Private Sub ErrorResponse__(ByVal uCalcHandle As IntPtr,ByVal Response As ErrorHandlerResponseEnum)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorSymbol")> _ 
   Private Function ErrorSymbol__(ByVal uCalcHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ErrorSetExpression")> _ 
   Private Sub ErrorSetExpression__(ByVal uCalcHandle As IntPtr,ByVal Expression As String)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Eval")> _ 
   Private Function Eval__(ByVal uCalcHandle As IntPtr,ByVal Expression As String) As  Double
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="EvalStr")> _ 
   Private Function EvalStr__(ByVal uCalcHandle As IntPtr,ByVal Expression As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Expand")> _ 
   Private Function Expand__(ByVal uCalcHandle As IntPtr,ByVal Expression As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ExpandSteps")> _ 
   Private Function ExpandSteps__(ByVal uCalcHandle As IntPtr,ByVal Expression As String , ByVal Separator As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvClearException")> _ 
   Private Function FPEnvClearException__(ByVal uCalcHandle As IntPtr, ByVal Exceptions As Int32  ) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvGetExceptionMask")> _ 
   Private Function FPEnvGetExceptionMask__(ByVal uCalcHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvRaiseException")> _ 
   Private Function FPEnvRaiseException__(ByVal uCalcHandle As IntPtr, ByVal Exception As FPEnvExceptionEnum  ) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvGetRounding")> _ 
   Private Function FPEnvGetRounding__(ByVal uCalcHandle As IntPtr) As  FPEnvRoundingEnum
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvSetExceptionMask")> _ 
   Private Sub FPEnvSetExceptionMask__(ByVal uCalcHandle As IntPtr,ByVal Exceptions As Int32)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvSetExceptionUnMask")> _ 
   Private Sub FPEnvSetExceptionUnMask__(ByVal uCalcHandle As IntPtr,ByVal Exceptions As Int32)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvSetRounding")> _ 
   Private Function FPEnvSetRounding__(ByVal uCalcHandle As IntPtr,ByVal Direction As FPEnvRoundingEnum) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="FPEnvToggleExceptionMask")> _ 
   Private Sub FPEnvToggleExceptionMask__(ByVal uCalcHandle As IntPtr,ByVal Exception As FPEnvExceptionEnum)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="HandleOf")> _ 
   Private Function HandleOf__(ByVal uCalcHandle As IntPtr,ByVal ItemName As String,  ByVal Properties As Int64 ,  ByVal SkipExpansion As Int32 ) As  IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ItemOf")> _ 
   Private Function ItemOf__(ByVal uCalcHandle As IntPtr,ByVal ItemName As String , ByVal Properties As Int64  , ByVal SkipExpansion As Int32  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ItemOfProperty")> _ 
   Private Function ItemOfProperty__(ByVal uCalcHandle As IntPtr,ByVal Property_ As ItemIsEnum , ByVal nth As Int32 , ByVal DataTypeName As String  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="License")> _ 
   Private Function License__(ByVal uCalcHandle As IntPtr,ByVal LicenseType As LicenseTypeEnum,  ByVal L1 As Int32 ,  ByVal L2 As Int32 ,  ByVal L3 As Int32 ,  ByVal L4 As Int32 ,  ByVal L5 As Int32 ) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NamespaceChild")> _ 
   Private Function NamespaceChild__(ByVal uCalcHandle As IntPtr, ByVal nth As Int32 ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NamespaceChildCount")> _ 
   Private Function NamespaceChildCount__(ByVal uCalcHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NamespaceParent")> _ 
   Private Function NamespaceParent__(ByVal uCalcHandle As IntPtr, ByVal Level As Int32 ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NamespaceSymbol")> _ 
   Private Function NamespaceSymbol__(ByVal uCalcHandle As IntPtr,ByVal nth As Int32) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="NamespaceSymbolCount")> _ 
   Private Function NamespaceSymbolCount__(ByVal uCalcHandle As IntPtr) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Parse")> _ 
   Private Function Parse__(ByVal uCalcHandle As IntPtr,ByVal Expression As String , ByVal DataType As String  ) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Precedence_Overload_uCalc")> _ 
   Private Function Precedence_Overload_uCalc__(ByVal uCalcHandle As IntPtr,ByVal Operator1 As String,  ByVal Operator2 As String ) As  Int32
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Properties")> _ 
   Private Function Properties__(ByVal uCalcHandle As IntPtr,ByVal p1 As ItemIsEnum,  ByVal p2 As ItemIsEnum ,  ByVal p3 As ItemIsEnum ,  ByVal p4 As ItemIsEnum ) As  Int64
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ReleaseuCalc")> _ 
   Private Sub ReleaseuCalc__(ByVal uCalcHandle As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ResetOutput")> _ 
   Private Sub ResetOutput__(ByVal uCalcHandle As IntPtr)
   End Sub

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Default")> _ 
   Private Function Default__(ByVal uCalcHandle As IntPtr) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetOutput_Overload_uCalc_Callback")> _ 
   Private Function SetOutput_Overload_uCalc_Callback__(ByVal uCalcHandle As IntPtr,ByVal FunctionAddress As uCalcDelegate , ByVal Properties As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="SetOutput_Overload_uCalc_NoCallback")> _ 
   Private Function SetOutput_Overload_uCalc_NoCallback__(ByVal uCalcHandle As IntPtr,ByVal Definition As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ValueAt")> _ 
   Private Function ValueAt__(ByVal uCalcHandle As IntPtr,ByVal ValuePtr As IntPtr, ByVal DataType As String) As IntPtr
   End Function

   <DllImport(uCalcDLL, CharSet:=CharSet.Ansi, CallingConvention:=CallingConvention.Cdecl, EntryPoint:="Handle_Overload_uCalc")> _ 
   Private Function Handle_Overload_uCalc__(ByVal uCalcHandle As IntPtr) As  IntPtr
   End Function

   ''' <summary> uCalc main class</summary>
   Public Class uCalc

      ''' <summary> Handle for uCalc instance</summary>
      Public  uCalcHandle As IntPtr 

      ''' <summary> Class for evaluating an already parsed expression or executing parsed code</summary>
      Public Class ParsedExpr

         Public ExprHandle As IntPtr 

         ''' <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed By Expression</summary>
         ''' <returns> The result of the evaluated floating point expression</returns>
         ''' <remarks><para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
         ''' is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
         ''' more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
         ''' going into the time-critical loop.  The ParsedExpr object returned by Parse is what calls the Evaluate function.
         ''' </para><para>
         ''' Evaluate can also be called by a ParsedExpr object that was passed to a callback By Expression.
         ''' </para><para>
         ''' The expression must be one who&#39;s evaluation returns a floating point numeric value.  If the expression evaluates
         ''' to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
         ''' type other than the default was explicitely specified with Parse, the value returned will be invalid.  However,
         ''' an error will not be raised.
         ''' </para></remarks>
         Function Evaluate() As  Double
            Return Evaluate__(ExprHandle)
         End Function

         ''' <summary> Rapidly evaluates expression that was parsed as integer with Parse or passed By Expression</summary>
         ''' <returns> The result of the evaluated integer expression</returns>
         ''' <remarks><para>
         ''' This is similar to Evaluate, but applies only to integer expressions.
         ''' </para></remarks>
         Function EvaluateInt32() As  Int32
            Return EvaluateInt32__(ExprHandle)
         End Function

         ''' <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed By Expression returning a string</summary>
         ''' <returns> The result of an evaluated expression of any data type, returned as a string</returns>
         ''' <remarks><para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
         ''' return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
         ''' before being returned.
         ''' </para></remarks>
         Function EvaluateStr() As String
            Return Marshal.PtrToStringAnsi(EvaluateStr__(ExprHandle)) 
         End Function

         ''' <summary> Rapidly evaluates an expression that was parsed with Parse or passed By Expression and returns a Pointer to the result</summary>
         ''' <returns> Returns a pointer to the result instead of the result itself</returns>
         ''' <remarks><para>Use this when you are interested in a pointer to the result instead of the result itself.  For instance, given an integer
         ''' result, you might use a pointer in order to retrieve the hi byte and lo byte values.
         ''' </para></remarks>
         Function EvaluateVoid() As  IntPtr
            Return EvaluateVoid__(ExprHandle)
         End Function

         ''' <summary> Rapidly executes code that was parsed with Parse or passed By Expression for which a result is not required</summary>
         ''' <remarks><para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
         ''' rather than a math expression.
         ''' </para></remarks>
         Sub Execute()
            Execute__(ExprHandle)
         End Sub

         ''' <summary> (Use Release instead)</summary>
         ''' <remarks><para>Use Release</para></remarks>
         Sub ReleaseExpr()
            ReleaseExpr__(ExprHandle)
         End Sub

         ''' <summary> Returns the handle of the parsed expression</summary>
         ''' <remarks><para>This numeric value represents the pointer that uCalc uses internally for the ParsedExpr object.
         ''' If you are iterating through a series of objects, such as with NextOverload, you may check Handle()
         ''' to see when it becomes NULL (or 0) to know when to stop.</para></remarks>
         Function Handle() As  IntPtr
            Return Handle_Overload_ParsedExpr__(ExprHandle)
         End Function

         ''' <summary> Releases an expression that was parsed with Parse</summary>
         ''' <remarks><para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
         ''' When the parsed expression is no longer needed it should be released with Release to preserve memory.
         ''' Note: An expression parsed with Parse must explicitely be released with Release unless it the uCalc
         ''' object it is in is released.  A parsed expression will not be released automatically when it simply
         ''' goes out of scope.
         ''' </para></remarks>
         Public Sub Release()
            ReleaseExpr()
            ExprHandle = Nothing 
         End Sub

         Public Sub New()
            ExprHandle = Nothing 
         End Sub

         Public Sub New(ByVal ExpressionHandle As IntPtr)
            ExprHandle = ExpressionHandle
         End Sub

         Public Sub New(ByVal ParsedExprInstance As ParsedExpr)
            ExprHandle = ParsedExprInstance.ExprHandle
         End Sub
      End Class

      ''' <summary> Class for handling uCalc items</summary>
      Public Class Item

         ''' <summary> Handle for a uCalc.Item</summary>
         Public  ItemHandle As IntPtr 

         ''' <summary> Returns the number of elements in a uCalc.Item</summary>
         ''' <remarks><para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
         ''' the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
         ''' the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.
         ''' </para><para>
         ''' When determining the number of elements, optional parameters in a function count as an element.  If the function
         ''' has a variable number of arguments, then count is -1.
         ''' </para><para>
         ''' The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
         ''' parts or some other complexity.
         ''' </para><para>
         ''' +++ the number of symbols in a uCalc
         ''' instance,</para></remarks>
         Function ElementCount() As  Int32
            Return ElementCount__(ItemHandle)
         End Function

         ''' <summary> Returns the data type of a uCalc.Item</summary>
         ''' <remarks><para>The returned data type object can be used to determine the type name, size, etc.</para></remarks>
         Function DataType() As DataType
            Return New DataType(DataType__(ItemHandle))
         End Function

         ''' <summary> Returns the callback address of a defined function (or operator)</summary>
         ''' <remarks><para>The returned value is the actual pointer address of the function as used by uCalc internally, and not
         ''' a delegate, even in .NET.  As such, in .NET this value can be used in the more general Define function but
         ''' not functions like DefineFunction or DefineneOperator.</para></remarks>
         Function FunctionAddress() As  IntPtr
            Return FunctionAddress__(ItemHandle)
         End Function

         ''' <summary> Determines if a uCalc.Item has the given properties</summary>
         ''' <param name="ItemProperty"> The property you want to check for</param>
         ''' <returns> Returns True if the uCalc.Item matches the given property, or False otherwise.</returns>
         ''' <remarks><para>This helps you determine whether the uCalc.Item is a function, operator, variable, data type,
         ''' literal value, etc, or has certain other special properties.  Note: Some items may have multiple
         ''' properties.  For instance the Function property may be set to true for arrays.
         ''' </para></remarks>
         Function IsProperty(ByVal ItemProperty As ItemIsEnum) As  Boolean
            Return  IsProperty__(ItemHandle,ItemProperty) <> 0 
         End Function

         ''' <summary> Returns the name of the uCalc.Item</summary>
         ''' <remarks><para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
         ''' That name can be retried with this function.</para></remarks>
         Function Name() As String
            Return Marshal.PtrToStringAnsi(Name_Overload_Item__(ItemHandle)) 
         End Function

         ''' <summary> Returns next uCalc.Item object that shares the same name</summary>
         ''' <returns> Returns next uCalc.Item object that shares the same name, or a NULL (or 0) if there is none</returns>
         ''' <remarks><para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
         ''' parameters but share the same name --, or if they (or variables) overshadow each other, this function returns
         ''' the next uCalc.Item object that shares the same name.</para></remarks>
         Function NextOverload() As Item
            Return New Item(NextOverload__(ItemHandle))
         End Function

         ''' <summary> Returns the precedence level of the uCalc.Item</summary>
         ''' <remarks><para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
         ''' may sometimes want to give it the same level as a pre-existing operator.</para></remarks>
         Function Precedence() As  Int32
            Return Precedence_Overload_Item__(ItemHandle)
         End Function

         ''' <summary> (Use Release instead)</summary>
         ''' <remarks><para>Use Release</para></remarks>
         Sub ReleaseItem()
            ReleaseItem__(ItemHandle)
         End Sub

         ''' <summary> Renames an item</summary>
         ''' <param name="NewName"> The new name of the item</param>
         ''' <remarks><para>
         ''' Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this function.
         ''' </para></remarks>
         Sub Rename(ByVal NewName As String)
            Rename__(ItemHandle,NewName)
         End Sub

         ''' <summary> Returns the replacement part of a syntax construct defined with DefineSyntax</summary>
         ''' <returns> Returns the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax</returns>
         ''' <remarks><para>This allows you to inspect the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax.</para></remarks>
         Function ReplacementText() As String
            Return Marshal.PtrToStringAnsi(ReplacementText__(ItemHandle)) 
         End Function

         ''' <summary> Sets the value of a floating point variable +++ maybe SetValue instead of SetVariableValue</summary>
         ''' <param name="Value"> Double precision floating point value that the variable will be set to</param>
         ''' <remarks><para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable using
         ''' SetVariableValue.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
         ''' to the address of a variable in your source code, and not have to use SetVariableValue.</para></remarks>
         Sub SetVariableValue(ByVal Value As Double)
            SetVariableValue__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a variable of any data type by passing a string</summary>
         ''' <param name="Value"> String containing the value that will be converted and stored in the variable</param>
         ''' <remarks><para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&#39;s data type.
         ''' The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
         ''' place with SetVariableValueAny, it is typically slower than calling SetVariableValue for floating point, or one of the
         ''' other versions of SetVariableValue specific to a data type.</para></remarks>
         Sub SetVariableValueAny(ByVal Value As String)
            SetVariableValueAny__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a Byte variable</summary>
         ''' <param name="Value"> Byte value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</para></remarks>
         Sub SetVariableValueByte(ByVal Value As Byte)
            SetVariableValueByte__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a 16-bit integer variable</summary>
         ''' <param name="Value"> 16-bit integer value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
         ''' both signed and unsigned 16-bit integers.</para></remarks>
         Sub SetVariableValueInt16(ByVal Value As Int16)
            SetVariableValueInt16__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a 32-bit integer variable</summary>
         ''' <param name="Value"> 32-bit integer value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
         ''' both signed and unsigned 32-bit integers.</para></remarks>
         Sub SetVariableValueInt32(ByVal Value As Int32)
            SetVariableValueInt32__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a 64-bit integer variable</summary>
         ''' <param name="Value"> 64-bit integer value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
         ''' both signed and unsigned 64-bit integers.</para></remarks>
         Sub SetVariableValueInt64(ByVal Value As Int64)
            SetVariableValueInt64__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a single precision floating point variable</summary>
         ''' <param name="Value"> Single precision value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para></remarks>
         Sub SetVariableValueSng(ByVal Value As Single)
            SetVariableValueSng__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a double precision floating point variable</summary>
         ''' <param name="Value"> Double precision floating point value that the variable will be set to</param>
         ''' <remarks><para>SetVariableValueDbl is synonymous with SetVariableValue</para></remarks>
         Sub SetVariableValueDbl(ByVal Value As Double)
            SetVariableValueDbl__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a string variable</summary>
         ''' <param name="Value"> String value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para></remarks>
         Sub SetVariableValueStr(ByVal Value As String)
            SetVariableValueStr__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a pointer variable</summary>
         ''' <param name="Value"> Pointer that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a pointer variable defined with DefineVariable (or Define).  Note: This does not
         ''' change the pointer of the variable.  It simply stores a pointer value in the variable.</para></remarks>
         Sub SetVariableValuePtr(ByVal Value As IntPtr)
            SetVariableValuePtr__(ItemHandle,Value)
         End Sub

         ''' <summary> Sets the value of a variable to a value who&#39;s Pointer is passed</summary>
         ''' <param name="Value"> Pointer to the value that the variable will be set to</param>
         ''' <remarks><para>This lets you set the value of a variable by passing the pointer to the value you want to set the variable to.</para></remarks>
         Sub SetVariableValueByPtr(ByVal Value As IntPtr)
            SetVariableValueByPtr__(ItemHandle,Value)
         End Sub

         ''' <summary> Returns the text data associated with a uCalc.Item</summary>
         ''' <remarks><para>Certain items, such as a syntax construct defined with DefineSyntax have a text property that can be retrieved
         ''' with a call to this function.</para></remarks>
         Function Text() As String
            Return Marshal.PtrToStringAnsi(Text__(ItemHandle)) 
         End Function

         ''' <summary> Returns the uCalc object that the uCalc.Item belongs to</summary>
         ''' <remarks><para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
         ''' the uCalc object the defined item belongs to.</para></remarks>
         Function uCalc() As uCalc
            Return New uCalc(uCalc_Overload_Item__(ItemHandle))
         End Function

         ''' <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Double precision value associated with a variable.</para></remarks>
         Function Value() As  Double
            Return Value__(ItemHandle)
         End Function

         ''' <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Byte value associated with a variable.</para></remarks>
         Function ValueByte() As  Byte
            Return ValueByte__(ItemHandle)
         End Function

         ''' <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Int16 value associated with a variable.</para></remarks>
         Function ValueInt16() As  Int16
            Return ValueInt16__(ItemHandle)
         End Function

         ''' <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Int32 value associated with a variable.</para></remarks>
         Function ValueInt32() As  Int32
            Return ValueInt32__(ItemHandle)
         End Function

         ''' <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Int64 value associated with a variable.</para></remarks>
         Function ValueInt64() As  Int64
            Return ValueInt64__(ItemHandle)
         End Function

         ''' <summary> Returns the Double precision value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Single precision value associated with a variable.</para></remarks>
         Function ValueSng() As  Single
            Return ValueSng__(ItemHandle)
         End Function

         ''' <summary> Returns the Double precision value associated with a uCalc.Item</summary>
         ''' <remarks><para>Synonymous with Value</para></remarks>
         Function ValueDbl() As  Double
            Return ValueDbl__(ItemHandle)
         End Function

         ''' <summary> Returns the String value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the String value associated with a variable.
         ''' </para><para>
         ''' ValueStr has a special property beyond the Value function for other data types in that it can return the value
         ''' of an item regardless of what data type it has.  The value is returned as a string.
         ''' </para></remarks>
         Function ValueStr() As String
            Return Marshal.PtrToStringAnsi(ValueStr__(ItemHandle)) 
         End Function

         ''' <summary> Returns the Pointer value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Pointer value associated with a variable.
         ''' Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         ''' that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         ''' ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any value you may
         ''' have set as the address the variable points to.</para></remarks>
         Function ValuePtr() As  IntPtr
            Return ValuePtr__(ItemHandle)
         End Function

         ''' <summary> Returns the Pointer address of the value associated with a uCalc.Item</summary>
         ''' <remarks><para>Use this to retrieve the Pointer address of the value associated with a variable (instead of
         ''' the value itself).  Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         ''' that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         ''' ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
         ''' you may have stored as the value of the variable.</para></remarks>
         Function ValueAddr() As  IntPtr
            Return ValueAddr__(ItemHandle)
         End Function

         ''' <summary> Returns the handle of the current uCalc.Item object</summary>
         Function Handle() As  IntPtr
            Return Handle_Overload_Item__(ItemHandle)
         End Function

         ''' <summary> Releases an Item defined with Define or other related function</summary>
         ''' <remarks><para>When a function, operator, variable, or other item is defined with such functions as
         ''' Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
         ''' released with this function.  Items should generally be released when they are no longer needed, in order
         ''' to preserve memory.
         ''' </para><para>
         ''' Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
         ''' object it is a part of is is released.  An Item object will not be released automatically when it
         ''' simply goes out of scope.
         ''' </para></remarks>
         Public Sub Release()
            ReleaseItem()
            ItemHandle = Nothing 
         End Sub

         Public Sub New()
            ItemHandle = Nothing 
         End Sub

         Public Sub New(ByVal PassedItemHandle As IntPtr)
            ItemHandle = PassedItemHandle
         End Sub

         Public Sub New(ByVal ItemHandleInstance As Item)
            ItemHandle = ItemHandleInstance.ItemHandle
         End Sub
      End Class

      ''' <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary>
      Public Class Callback

         ''' <summary> Handle of the expression part the current callback instance is refering to</summary>
         Public  ExprPartHandle As IntPtr 

         ''' <summary> Returns the value of an argument passed to a uCalc.Callback function</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>When the name of a callback function is specified in DefineFunction or DefineOperator, that
         ''' callback function will receive arguments uCalc passes to it through the Arg.  Arg handles values
         ''' of the default Double floating point data type.  For other data types, ArgInt32, ArgStr, etc instead.
         ''' </para><para>
         ''' Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
         ''' 1 (not 0), and the second and third will be 2 and 3 respectively.
         ''' </para><para>
         ''' If you are defining a binary operator, 1 is the index for the left operand, and 2 for the right operand.
         ''' For a unary operator, the index is 1.
         ''' </para></remarks>
         Function Arg(ByVal Index As Int32) As  Double
            Return Arg__(ExprPartHandle,Index)
         End Function

         ''' <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary>
         ''' <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
         ''' <remarks><para>This is useful for functions that have a variable number of arguments</para></remarks>
         Function ArgCount() As  Int32
            Return ArgCount__(ExprPartHandle)
         End Function

         ''' <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <returns> The returned object can be used (where relevent) just like any other uCalc.Item</returns>
         ''' <remarks><para>Each argument is a uCalc.Item, just like anything else defined with Define.  With this object,
         ''' you can do such things as determine the argument&#39;s data type, determine which instance of uCalc
         ''' it belongs to, retrieve the pointer of the value of the argument, etc.</para></remarks>
         Function ArgObj(ByVal Index As Int32) As Item
            Return New Item(ArgObj__(ExprPartHandle,Index))
         End Function

         ''' <summary> Returns the address of the argument value that is being passed</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>In addition to Arg which returns arguments that are of the default Double floating point numeric type, there
         ''' are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
         ''' a data type that does not have a specialized function, you can retrieve the address of the argument value as
         ''' a pointer, and use that to retrieve the value.</para></remarks>
         Function ArgAddr(ByVal Index As Int32) As  IntPtr
            Return ArgAddr__(ExprPartHandle,Index)
         End Function

         ''' <summary> Returns the object instance of the expression that&#39;s passed as an argument to a uCalc.Callback function+++Example</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <returns> Returns parsed expression object with which you can call functions like Evaluate or Execute</returns>
         ''' <remarks><para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
         ''' by expression.  When an expression is passed By Expression, the expresison is not evaluated before being passed.
         ''' Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.</para></remarks>
         Function ArgExpr(ByVal Index As Int32) As ParsedExpr
            Return New ParsedExpr(ArgExpr__(ExprPartHandle,Index))
         End Function

         ''' <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
         ''' recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
         ''' place first before being passed.</para></remarks>
         Function ArgInt32(ByVal Index As Int32) As  Int32
            Return ArgInt32__(ExprPartHandle,Index)
         End Function

         ''' <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can
         ''' recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
         ''' place first before being passed.</para></remarks>
         Function ArgInt64(ByVal Index As Int32) As  Int64
            Return ArgInt64__(ExprPartHandle,Index)
         End Function

         ''' <summary> Returns a pointer value being passed as an argument to a uCalc.Callback function</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>If a function parameter (or operator operand) was explicitely defined as a Pointer, then the callback function can
         ''' recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
         ''' address of the value being passed.</para></remarks>
         Function ArgPtr(ByVal Index As Int32) As  IntPtr
            Return ArgPtr__(ExprPartHandle,Index)
         End Function

         ''' <summary> Returns the value being passed as argument in to a uCalc.Callback function in form of a string</summary>
         ''' <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         ''' <remarks><para>In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
         ''' was passed as any data type.  If the argument was not a string, ArgStr converts it to a string before returning its value.</para></remarks>
         Function ArgStr(ByVal Index As Int32) As String
            Return Marshal.PtrToStringAnsi(ArgStr__(ExprPartHandle,Index)) 
         End Function

         ''' <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary>
         ''' <param name="Value"> Floating point value that your callback returns</param>
         ''' <remarks><para>When you define a callback function with DefineFunction and you want to return a floating point number,
         ''' do it with Return.
         ''' </para><para>
         ''' Note: For compilers in which Return is a reserved keyword, use ReturnDbl instead.
         ''' </para></remarks>
         Sub ReturnDbl(ByVal Value As Double)
            Return__(ExprPartHandle,Value)
         End Sub

         ''' <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
         ''' <param name="Value"> Integer (32-bit) that your callback returns</param>
         ''' <remarks><para>When you define a callback function with DefineFunction and you want to return a Int32,
         ''' do it with ReturnInt32.</para></remarks>
         Sub ReturnInt32(ByVal Value As Int32)
            ReturnInt32__(ExprPartHandle,Value)
         End Sub

         ''' <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
         ''' <param name="Value"> Integer (64-bit) that your callback returns</param>
         ''' <remarks><para>When you define a callback function with DefineFunction and you want to return a Int64,
         ''' do it with ReturnInt64.</para></remarks>
         Sub ReturnInt64(ByVal Value As Int64)
            ReturnInt64__(ExprPartHandle,Value)
         End Sub

         ''' <summary> Sets the return value of a uCalc.Callback function as a Pointer</summary>
         ''' <param name="Value"> Pointer value that your callback returns</param>
         ''' <remarks><para>When you define a callback function with DefineFunction that returns a pointer value,
         ''' do it with ReturnPtr.</para></remarks>
         Sub ReturnPtr(ByVal Value As IntPtr)
            ReturnPtr__(ExprPartHandle,Value)
         End Sub

         ''' <summary> Sets the return value of a uCalc.Callback function as a String</summary>
         ''' <param name="Value"> String value that your callback returns</param>
         ''' <remarks><para>When you define a callback function with DefineFunction that returns a string,
         ''' do it with ReturnStr.</para></remarks>
         Sub ReturnStr(ByVal Value As String)
            ReturnStr__(ExprPartHandle,Value)
         End Sub

         ''' <summary> Raises an error in a uCalc.Callback function</summary>
         ''' <param name="ErrNumber"> Index number of error to raise</param>
         ''' <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
         ''' <remarks><para>Your callback can raise an error by calling this funciton with one of the values of the
         ''' ErrorNumberEnum enumerator.  End-user function definitions can also raise an error by calling Error().
         ''' The return value is determined by error handlers that you may have added.  ErrorAbort (value of 0)
         ''' is returned by default if there are no error handlers.</para></remarks>
         Function ErrorRaise(ByVal ErrNumber As ErrorNumberEnum) As  Int32
            Return ErrorRaise__(ExprPartHandle,ErrNumber)
         End Function

         ''' <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary>
         ''' <param name="ErrMessage"> Text of custom error message to return</param>
         ''' <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
         ''' <remarks><para>Your callback can raise a custom error message (one not listed as a ErrorNumberEnum enumerator)
         ''' by simply passing the text of the message to this function.</para></remarks>
         Function ErrorRaiseMessage(ByVal ErrMessage As String) As  Int32
            Return ErrorRaiseMessage__(ExprPartHandle,ErrMessage)
         End Function

         ''' <summary> Returns the uCalc instance the current expression part belongs to</summary>
         ''' <remarks><para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
         ''' which particular instance of uCalc the expression part being passed belongs to.</para></remarks>
         Function uCalc() As uCalc
            Return New uCalc(uCalc_Overload_Callback__(ExprPartHandle))
         End Function

         ''' <summary> Returns the handle of the sub expression passed to the callback</summary>
         ''' <remarks><para>This pointer value is the argument received by a callback function associated with a Function Or_  operator
         ''' defined with DefineFunction or DefineOperator.  Once a uCalc.Callback object is construct with this handle, this
         ''' handle typically no longer needs to be refered to directly.  However, it&#39;s still avaible should you need it.
         ''' uCalc interfaces with this handle underneath the hood of the uCalc.Callback object.
         ''' </para></remarks>
         Function Handle() As  IntPtr
            Return Handle_Overload_Callback__(ExprPartHandle)
         End Function

         ''' <summary> Returns the Function Or_  operator Item of the current expression part</summary>
         ''' <remarks><para>If several functions and/or operators are mapped to the same callback function, you can use this to
         ''' let you determine which one is currently being called, based on properties you can check.</para></remarks>
         Function OpItem() As Item
            Return New Item(OpItem__(ExprPartHandle))
         End Function

         Public Sub New()
            ExprPartHandle = Nothing 
         End Sub

         Public Sub New(ByVal ExpressionHandle As IntPtr)
            ExprPartHandle = ExpressionHandle
         End Sub

         Public Sub New(ByVal ExprInstance As Callback)
            ExprPartHandle = ExprInstance.ExprPartHandle
         End Sub
      End Class

      ''' <summary> Class for working with uCalc data types</summary>
      Public Class DataType

         ''' <summary> Handle for uCalc Data Type</summary>
         Public  DataTypeHandle As IntPtr 

         ''' <summary> Allocates an array of the given data type</summary>
         ''' <param name="ArraySize"> Size - in umber of elements - of the array you&#39;re allocating</param>
         ''' <returns> Pointer of the array that was allocated</returns>
         ''' <remarks><para>This allocates an array of the given type.  Elements of the array are set to the default value
         ''' of that type.</para></remarks>
         Function AllocArray(ByVal ArraySize As Int32) As  IntPtr
            Return AllocArray__(DataTypeHandle,ArraySize)
         End Function

         ''' <summary> Allocates a scalar of the given type and copies data from the area indicated by the pointer</summary>
         ''' <param name="ValuePtr"> Pointer to the data that is copied to the scalar being defined</param>
         ''' <returns> Pointer of the scalar that was allocated</returns>
         Function AllocByPtr(ByVal ValuePtr As IntPtr) As  IntPtr
            Return AllocByPtr__(DataTypeHandle,ValuePtr)
         End Function

         ''' <summary> Allocates a scalar of the given data type</summary>
         ''' <param name="Value"> Expression who&#39;s evaluated value will be copied to the allocated scalar</param>
         ''' <returns> Pointer of the scalar that was allocated</returns>
         Function AllocScalar(Optional  ByVal Value As String = "") As  IntPtr
            Return AllocScalar__(DataTypeHandle,Value)
         End Function

         ''' <summary> Returns the size (in bytes) of one unit of data for this data type</summary>
         Function ByteSize() As  Int32
            Return ByteSize__(DataTypeHandle)
         End Function

         ''' <summary> Returns the index of the data type</summary>
         ''' <remarks><para>The return value is a BuiltInTypesEnum enumerator value that represents the data type of the
         ''' current object.  For instance, if the data type is a string, the return value would be Type_String.
         ''' </para></remarks>
         Function Index() As  BuiltInTypesEnum
            Return Index__(DataTypeHandle)
         End Function

         ''' <summary> Returns the uCalc.Item handle of the data type</summary>
         ''' <remarks><para>Each data type object can also be represented as a uCalc object by using this return value.</para></remarks>
         Function Item() As Item
            Return New Item(Item_Overload_DataType__(DataTypeHandle))
         End Function

         ''' <summary> Returns the name of the data type</summary>
         Function Name() As String
            Return Marshal.PtrToStringAnsi(Name_Overload_DataType__(DataTypeHandle)) 
         End Function

         ''' <summary> Releases an array that was allocated with AllocArray</summary>
         ''' <param name="ValuePtr"> Pointer of uCalc array to be released from memory</param>
         ''' <remarks><para>This should only be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
         Sub ReleaseArray(ByVal ValuePtr As IntPtr)
            ReleaseArray__(DataTypeHandle,ValuePtr)
         End Sub

         ''' <summary> Releases a scalar that was allocated with AllocScalar</summary>
         ''' <param name="ValuePtr"> Pointer of uCalc scalar to be released from memory</param>
         ''' <remarks><para>This should only be used for scalars that were explicitly allocated with uCalc&#39;s AllocScalar.</para></remarks>
         Sub ReleaseScalar(ByVal ValuePtr As IntPtr)
            ReleaseScalar__(DataTypeHandle,ValuePtr)
         End Sub

         ''' <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         ''' <param name="ValuePtr"> Pointer of the uCalc array to be reset</param>
         ''' <remarks><para>Resets all the elements of the array to the default value for the given data type.  This should only
         ''' be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
         Sub ResetArray(ByVal ValuePtr As IntPtr)
            ResetArray__(DataTypeHandle,ValuePtr)
         End Sub

         ''' <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         ''' <param name="ValuePtr"> Pointer of the uCalc scalar to be reset</param>
         Sub ResetScalar(ByVal ValuePtr As IntPtr)
            ResetScalar__(DataTypeHandle,ValuePtr)
         End Sub

         ''' <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
         ''' <param name="Destination"> Pointer of array (base address) who&#39;s element value is to be set</param>
         ''' <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         ''' <param name="Index"> Index of the array element who&#39;s value needs to be set</param>
         Sub SetArray(ByVal Destination As IntPtr , ByVal Source As IntPtr , ByVal Index As Int32)
            SetArray__(DataTypeHandle,Destination , Source , Index)
         End Sub

         ''' <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
         ''' <param name="Destination"> Pointer of scalar who&#39;se value is to be set</param>
         ''' <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         Sub SetScalar(ByVal Destination As IntPtr , ByVal Source As IntPtr)
            SetScalar__(DataTypeHandle,Destination , Source)
         End Sub

         ''' <summary> swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
         ''' <param name="ValuePtrA"> Pointer of variable 1</param>
         ''' <param name="ValuePtrB"> Pointer of variable 2</param>
         Sub Swap(ByVal ValuePtrA As IntPtr , ByVal ValuePtrB As IntPtr)
            Swap__(DataTypeHandle,ValuePtrA , ValuePtrB)
         End Sub

         ''' <summary> Retuns a string representing the value at the given address formatted for the given data type</summary>
         ''' <param name="ValuePtr"> Pointer of the data who&#39;s value you want to display</param>
         ''' <remarks><para>If you have a pointer to data of the given type, you can display this data as a string with this function.  Special formatting
         ''' associated with a data type is used, if such formatting was defined.  For instance, if you&#39;re dealing with a Boolean, then
         ''' you might get: True, Yes, On, etc, depending on how the formatting was configured with SetOutput.</para></remarks>
         Shadows Function ToString(ByVal ValuePtr As IntPtr) As String
            Return Marshal.PtrToStringAnsi(ToString_Overload_DataType_ByPtr__(DataTypeHandle,ValuePtr)) 
         End Function

         ''' <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary>
         ''' <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
         ''' <remarks><para>If you pass a string, it will be converted to the given data type, and return using the formatting
         ''' for that data type.</para></remarks>
         Shadows Function ToString(ByVal Value As String) As String
            Return Marshal.PtrToStringAnsi(ToString_Overload_DataType_ByStr__(DataTypeHandle,Value)) 
         End Function

         ''' <summary> Returns the handle of the data type object</summary>
         ''' <remarks><para>Generally for internal use</para></remarks>
         Function Handle() As  IntPtr
            Return Handle_Overload_DataType__(DataTypeHandle)
         End Function

         Public Sub New()
            DataTypeHandle = Nothing 
         End Sub

         Public Sub New(ByVal PassedDataTypeHandle As IntPtr)
            DataTypeHandle = PassedDataTypeHandle
         End Sub

         Public Sub New(ByVal DataTypeHandleInstance As DataType)
            DataTypeHandle = DataTypeHandleInstance.DataTypeHandle
         End Sub
      End Class

      ''' <summary> Adds an error handler</summary>
      ''' <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
      ''' <param name="Position"> Position in which to insert the error handler</param>
      ''' <returns> The error handler can later be removed using this return object</returns>
      ''' <remarks><para>Use this to set a a callback function that is called whenever there is an error.  Your callback can
      ''' do things like alert the user of the error, correct the error and resume, keep a log of errors, etc.
      ''' </para><para>
      ''' By default, if there are multiple error handlers, they are called starting with the most recently
      ''' defined one first, ending with the first one to be defined last.  In order to define a handler in a different
      ''' order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
      ''' is called last, or a positive integer indicated where it should be inserted.  If the Postion value is out
      ''' of bounds, it is adjusted into the maximum acceptable value.
      ''' </para><para>
      ''' If you set ErrorResponse to ErrorHandlerResponseEnum.ErrorResume, subsequent error handler callbacks will
      ''' be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
      ''' that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.
      ''' </para><para>
      ''' Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
      ''' during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
      ''' </para><para>
      ''' For Floating Point Environment errors, such as: Err_Float_Division_By_Zero, Err_Float_Inexact_Result,
      ''' Err_Float_Invalid_Operation, Err_Float_Overflow, and Err_Float_Underflow, the uCalc error handler callbacks
      ''' are called only if the corresponding environment bits are not masked.
      ''' </para><para>
      ''' </para></remarks>
      Function AddErrorHandler(ByVal ErrorHandlerAddress As uCalcDelegate ,Optional  ByVal Position As Int32  = 0) As Item
         Return New Item(AddErrorHandler__(uCalcHandle,ErrorHandlerAddress ,Position))
      End Function

      ''' <summary> Creates a new uCalc instance</summary>
      ''' <remarks><para>This is not meant to be called directly.</para></remarks>
      Function CreateuCalc() As  IntPtr
         Return CreateuCalc__(uCalcHandle)
      End Function

      ''' <summary> Returns a data type object</summary>
      ''' <param name="Index"> Index associated with the data type</param>
      ''' <remarks><para>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For
      ''' instance, ItemOf(&quot;Int64&quot;) in one instance of uCalc does not share the same handle as ItemOf(&quot;Int64&quot;) in
      ''' another instance.  This allows the same data type to have a different name and different formatting in
      ''' different instances.  However, they share the same data type index across instances.  Given the index
      ''' of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
      ''' faster than obtaining it by name as text with ItemOf.</para></remarks>
      Function DataTypeOfIndex(ByVal Index As BuiltInTypesEnum) As DataType
         Return New DataType(DataTypeOfIndex__(uCalcHandle,Index))
      End Function

      ''' <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
      ''' <param name="Expression"> Text containing item to be defined.  Expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression</param>
      ''' <param name="Address"> Address a Function Or_  variable is attached to</param>
      ''' <param name="ExistingItem"> Address of existing item this definition is associated with</param>
      ''' <remarks><para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
      ''' this method.  Although there are specialized methods that do the same things, such as DefineFunction,
      ''' DefineOperator, DefineVariable, DefineConstant, and DefineSyntax, this method may in same cases offer
      ''' more flexibility.
      ''' </para><para>
      ''' The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
      ''' see Definition Keywords.
      ''' </para><para>
      ''' Sections are separated by ~~ (two tilda signs).
      ''' </para></remarks>
      Function Define(ByVal Expression As String ,Optional  ByVal Address As IntPtr = Nothing,Optional  ByVal ExistingItem As IntPtr = Nothing) As Item
         Return New Item(Define__(uCalcHandle,Expression ,Address,ExistingItem))
      End Function

      ''' <summary> Defines a constant</summary>
      ''' <param name="Definition"> Text containing constant to be defined</param>
      ''' <param name="VariableAddress"> Address the constant is attached to</param>
      ''' <remarks><para>
      ''' A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
      ''' You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
      ''' name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
      ''' the end-user attempts to assign a value to the constant, an error is raised.
      ''' </para><para>
      ''' DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)
      ''' </para></remarks>
      Function DefineConstant(ByVal Definition As String ,Optional  ByVal VariableAddress As IntPtr = Nothing) As Item
         Return New Item(DefineConstant__(uCalcHandle,Definition ,VariableAddress))
      End Function

      ''' <summary> Defines a data type</summary>
      ''' <param name="TypeSize"> Number of bytes 1 scalar value of this type takes</param>
      ''' <param name="AllocArrayValue"> Address of a function that will allocate an array of this type</param>
      ''' <param name="AllocScalar"> Address of a function that will allocate a scalar of this type.  If a string is passed, this function must convert it to this type and store the value, otherwise the function must store a default value for this type.</param>
      ''' <param name="AllocScalarByPtr"> Address of a function that takes a pointer to a value of this type, allocates a scalar and stores the copied value in the allocated space.</param>
      ''' <param name="DeleteArray"> Address of a function that frees a memory block that was allocated with AllocValueArray</param>
      ''' <param name="DeleteScalar"> Address of a function that frees a memory block that was allocated by AllocScalar or AllocScalarByPtr.</param>
      ''' <param name="ResetScalar"> Address of a function that resets an allocated scalar back to the default value of this type.</param>
      ''' <param name="ResetArray"> Address of a function that resets array elements back to the default value of this type.</param>
      ''' <param name="SetValueArray"> Address of a function that stores an array element value.</param>
      ''' <param name="SetValueScalar"> Address of a function that stores a scalar value.</param>
      ''' <param name="SwapValues"> Address of a function that swaps two values of this type.</param>
      ''' <param name="ToString"> Address of a function that converts this data type to a string.</param>
      ''' <remarks><para>
      ''' This allows you to define a data type that functions at the same level as the intrinsic types that come with uCalc.
      ''' </para></remarks>
      Function DefineDataType(ByVal TypeSize As Int32 , ByVal AllocArrayValue As IntPtr , ByVal AllocScalar As IntPtr , ByVal AllocScalarByPtr As IntPtr , ByVal DeleteArray As IntPtr , ByVal DeleteScalar As IntPtr , ByVal ResetScalar As IntPtr , ByVal ResetArray As IntPtr , ByVal SetValueArray As IntPtr , ByVal SetValueScalar As IntPtr , ByVal SwapValues As IntPtr , ByVal ToString As IntPtr) As Item
         Return New Item(DefineDataType__(uCalcHandle,TypeSize , AllocArrayValue , AllocScalar , AllocScalarByPtr , DeleteArray , DeleteScalar , ResetScalar , ResetArray , SetValueArray , SetValueScalar , SwapValues , ToString))
      End Function

      ''' <summary> Defines a function</summary>
      ''' <param name="Definition"> Text containing function to be defined</param>
      ''' <param name="FunctionAddress"> Address of callback function this function is attached to</param>
      ''' <remarks><para>
      ''' A function definition can either be self-contained, or it may be attached to the address of a callback function in your
      ''' source code.
      ''' </para><para>
      ''' DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)
      ''' </para></remarks>
      Function DefineFunction(ByVal Definition As String ,Optional  ByVal FunctionAddress As uCalcDelegate = Nothing) As Item
         Return New Item(DefineFunction__(uCalcHandle,Definition ,FunctionAddress))
      End Function

      ''' <summary> Defines an operator</summary>
      ''' <param name="Definition"> Text containing operator to be defined</param>
      ''' <param name="iPrecedence"> Precedence level of the operator.</param>
      ''' <param name="Grouping"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
      ''' <param name="FunctionAddress"> Address of callback function this operator is attached to</param>
      ''' <remarks><para>
      ''' Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
      ''' defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
      ''' and select a precedence level in comparison to those operators.  By default operators are grouped left to right.  But you
      ''' may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
      ''' the right will be evaluated first.
      ''' </para><para>
      ''' DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)
      ''' More later+++
      ''' </para></remarks>
      Function DefineOperator(ByVal Definition As String , ByVal iPrecedence As Int32 ,Optional  ByVal Grouping As GroupingEnum = GroupingEnum.GroupLeftToRight,Optional  ByVal FunctionAddress As uCalcDelegate = Nothing) As Item
         Return New Item(DefineOperator__(uCalcHandle,Definition , iPrecedence ,Grouping,FunctionAddress))
      End Function

      ''' <summary> Defines a syntax construct</summary>
      ''' <param name="Definition"> Text containing syntax construct</param>
      Function DefineSyntax(ByVal Definition As String) As Item
         Return New Item(DefineSyntax__(uCalcHandle,Definition))
      End Function

      ''' <summary> Defines a transform</summary>
      ''' <param name="FromExpr"> Pattern representing the text that must be transformed</param>
      ''' <param name="ToExpr"> The text that the matching test is transformed to</param>
      ''' <param name="ItemHandle"> Handle</param>
      Function DefineTransform(ByVal FromExpr As String , ByVal ToExpr As String ,Optional  ByVal ItemHandle As IntPtr = Nothing) As Item
         Return New Item(DefineTransform__(uCalcHandle,FromExpr , ToExpr ,ItemHandle))
      End Function

      ''' <summary> Defines a variable</summary>
      ''' <param name="Definition"> Text containing variable to be defined</param>
      ''' <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
      ''' <remarks><para>
      ''' A variable consists of ...
      ''' </para><para>
      ''' DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)
      ''' </para></remarks>
      Function DefineVariable(ByVal Definition As String ,Optional  ByVal VariableAddress As IntPtr = Nothing) As Item
         Return New Item(DefineVariable__(uCalcHandle,Definition ,VariableAddress))
      End Function

      ''' <summary> Returns the expression associated with an error that was raised</summary>
      Function ErrorExpression() As String
         Return Marshal.PtrToStringAnsi(ErrorExpression__(uCalcHandle)) 
      End Function

      ''' <summary> Returns the location of an error that was raised</summary>
      Function ErrorLocation() As  Int32
         Return ErrorLocation__(uCalcHandle)
      End Function

      ''' <summary> Returns a message associated with the error that was raised</summary>
      ''' <param name="Index"> Index of error message to return</param>
      ''' <remarks><para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
      ''' then the error message associated with this Index is returned.  A value must be chosen from ErrorNumberEnum.
      ''' </para><para>
      ''' Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      ''' </para></remarks>
      Function ErrorMessage(Optional  ByVal Index As ErrorNumberEnum = ErrorNumberEnum.Err_None) As String
         Return Marshal.PtrToStringAnsi(ErrorMessage__(uCalcHandle,Index)) 
      End Function

      ''' <summary> Returns the error number of the error that was raised</summary>
      ''' <remarks><para>
      ''' Returns the error number of the most recently raised error.
      ''' </para><para>
      ''' Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      ''' </para></remarks>
      Function ErrorNumber() As  ErrorNumberEnum
         Return ErrorNumber__(uCalcHandle)
      End Function

      ''' <summary> Sets the error response for an error handler</summary>
      ''' <param name="Response"> Indicates what should happen now that an error is raised</param>
      ''' <remarks><para>When an error is raised, you have the option of aborting the rest of the option (which is the default option),
      ''' or you may correct the problem, and then resume the operation.  Or you may perform an action, and than raise the
      ''' error again for a subsequent error handle to take care of.
      ''' </para><para>
      ''' This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
      ''' The options to use can be found at ErrorHandlerResponseEnum.
      ''' </para></remarks>
      Sub ErrorResponse(ByVal Response As ErrorHandlerResponseEnum)
         ErrorResponse__(uCalcHandle,Response)
      End Sub

      ''' <summary> Returns the symbol associated with the error that is raised</summary>
      Function ErrorSymbol() As String
         Return Marshal.PtrToStringAnsi(ErrorSymbol__(uCalcHandle)) 
      End Function

      ''' <summary> Sets a new expression after an error is raised</summary>
      ''' <param name="Expression"> Error expression</param>
      Sub ErrorSetExpression(ByVal Expression As String)
         ErrorSetExpression__(uCalcHandle,Expression)
      End Sub

      ''' <summary> Parses and evaluates an expression in one step</summary>
      ''' <param name="Expression"> Expression to be evaluated</param>
      ''' <remarks><para>This takes a math expression and returns a numeric result.</para></remarks>
      Function Eval(ByVal Expression As String) As  Double
         Return Eval__(uCalcHandle,Expression)
      End Function

      ''' <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
      ''' <param name="Expression"> Expression to be evaluated</param>
      ''' <remarks><para>This is similar to Eval, however, it can return non-numeric results, such as strings.</para></remarks>
      Function EvalStr(ByVal Expression As String) As String
         Return Marshal.PtrToStringAnsi(EvalStr__(uCalcHandle,Expression)) 
      End Function

      ''' <summary> Expands an expression</summary>
      ''' <param name="Expression"> Expression to be expanded</param>
      Function Expand(ByVal Expression As String) As String
         Return Marshal.PtrToStringAnsi(Expand__(uCalcHandle,Expression)) 
      End Function

      ''' <summary> Returns each step leading to an expression expansion</summary>
      ''' <param name="Expression"> Expression to be expanded</param>
      ''' <param name="Separator"> Separator used between steps</param>
      Function ExpandSteps(ByVal Expression As String ,Optional  ByVal Separator As String = "") As String
         Return Marshal.PtrToStringAnsi(ExpandSteps__(uCalcHandle,Expression ,Separator)) 
      End Function

      ''' <summary> Clears floating point exception(s) that were raised</summary>
      ''' <param name="Exceptions"> Exceptions</param>
      Function FPEnvClearException(Optional  ByVal Exceptions As Int32  = 0) As  Int32
         Return FPEnvClearException__(uCalcHandle,Exceptions)
      End Function

      ''' <summary> Returns the floating point environment error mask</summary>
      Function FPEnvGetExceptionMask() As  Int32
         Return FPEnvGetExceptionMask__(uCalcHandle)
      End Function

      ''' <summary> Raises a floating point exception in a uCalc.Callback function</summary>
      ''' <param name="Exception"> Bitmask value consisting of one or more exception flags to raise</param>
      ''' <returns> Returns a 0 if all the specified execptions were raised successfully, otherwise a non-zero value is returned.</returns>
      ''' <remarks><para>This attempts to raise the exception(s) specfied by the Exception parameter.  This is based on the C++
      ''' feraiseexcept() function.  If you are using C++, you may use the pre-defined macro values of 8,
      ''' 1, 16, 4, 2, 31.  If you are using another compiler, you may
      ''' use the matching members of the FPEnvExceptionEnum enumerator.
      ''' </para></remarks>
      Function FPEnvRaiseException(Optional  ByVal Exception As FPEnvExceptionEnum = FPEnvExceptionEnum.Invalid) As  Int32
         Return FPEnvRaiseException__(uCalcHandle,Exception)
      End Function

      ''' <summary> Gets the current floating point environment rounding option</summary>
      Function FPEnvGetRounding() As  FPEnvRoundingEnum
         Return FPEnvGetRounding__(uCalcHandle)
      End Function

      ''' <summary> Sets the current floating point environment mask</summary>
      Sub FPEnvSetExceptionMask(ByVal Exceptions As Int32)
         FPEnvSetExceptionMask__(uCalcHandle,Exceptions)
      End Sub

      ''' <summary> Unmasks the current floating point environment mask</summary>
      Sub FPEnvSetExceptionUnMask(ByVal Exceptions As Int32)
         FPEnvSetExceptionUnMask__(uCalcHandle,Exceptions)
      End Sub

      ''' <summary> Sets the floating point environment rounding option</summary>
      Function FPEnvSetRounding(ByVal Direction As FPEnvRoundingEnum) As  Int32
         Return FPEnvSetRounding__(uCalcHandle,Direction)
      End Function

      ''' <summary> Toggles the floating point environment mask</summary>
      Sub FPEnvToggleExceptionMask(ByVal Exception As FPEnvExceptionEnum)
         FPEnvToggleExceptionMask__(uCalcHandle,Exception)
      End Sub

      ''' <summary> Returns the handle of a uCalc.Item given a name</summary>
      Function HandleOf(ByVal ItemName As String, Optional  ByVal Properties As Int64 = 0, Optional  ByVal SkipExpansion As Boolean = False ) As  IntPtr
         Return HandleOf__(uCalcHandle,ItemName, Properties, CInt(SkipExpansion))
      End Function

      ''' <summary> Returns the uCalc.Item object associated with the given name</summary>
      ''' <param name="ItemName"> Name of uCalc.Item that you want to get</param>
      ''' <param name="Properties"> Properties to help pinpoint the right Item, in case several items share the same name</param>
      ''' <param name="SkipExpansion"> Some items are define as a syntax construct alias of another item.  By default the symbol is expanded first.  However, use this option if the alias is the object you&#39;re actually after.</param>
      ''' <remarks><para>
      ''' If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
      ''' </para><para>
      ''' In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
      ''' binary and unary versions.  You can specify Prefix if you wanted the unary one.
      ''' </para><para>
      ''' Note: A ItemIsEnum member should not be passed directly as the second argument.  Instead Properties, with one or more
      ''' ItemIsEnum members should be passed.
      ''' </para></remarks>
      Function ItemOf(ByVal ItemName As String ,Optional  ByVal Properties As Int64  = 0,Optional  ByVal SkipExpansion As Boolean = False ) As Item
         Return New Item(ItemOf__(uCalcHandle,ItemName ,Properties,CInt(SkipExpansion)))
      End Function

      ''' <summary> Returns the nth uCalc.Item object that matches a property</summary>
      ''' <param name="Property_"> Property of the uCalc.Item object you&#39;re after</param>
      ''' <param name="nth"> The nth object</param>
      ''' <param name="DataTypeName"> The data type of the object</param>
      ''' <remarks><para>
      ''' This can be used to cycle through defined items.
      ''' If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
      ''' </para></remarks>
      Function ItemOfProperty(ByVal Property_ As ItemIsEnum , ByVal nth As Int32 ,Optional  ByVal DataTypeName As String = "") As Item
         Return New Item(ItemOfProperty__(uCalcHandle,Property_ , nth ,DataTypeName))
      End Function

      ''' <summary> Sets the license code</summary>
      Function License(ByVal LicenseType As LicenseTypeEnum, Optional  ByVal L1 As Int32 = 0, Optional  ByVal L2 As Int32 = 0, Optional  ByVal L3 As Int32 = 0, Optional  ByVal L4 As Int32 = 0, Optional  ByVal L5 As Int32 = 0) As  Int32
         Return License__(uCalcHandle,LicenseType, L1, L2, L3, L4, L5)
      End Function

      ''' <summary> ???</summary>
      Function NamespaceChild(Optional  ByVal nth As Int32 = 0) As Item
         Return New Item(NamespaceChild__(uCalcHandle,nth))
      End Function

      ''' <summary> ???</summary>
      Function NamespaceChildCount() As  Int32
         Return NamespaceChildCount__(uCalcHandle)
      End Function

      ''' <summary> ???</summary>
      Function NamespaceParent(Optional  ByVal Level As Int32 = 0) As Item
         Return New Item(NamespaceParent__(uCalcHandle,Level))
      End Function

      ''' <summary> ???</summary>
      Function NamespaceSymbol(ByVal nth As Int32) As Item
         Return New Item(NamespaceSymbol__(uCalcHandle,nth))
      End Function

      ''' <summary> ???</summary>
      Function NamespaceSymbolCount() As  Int32
         Return NamespaceSymbolCount__(uCalcHandle)
      End Function

      ''' <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      ''' <param name="Expression"> Expression to be parsed</param>
      ''' <param name="DataType"> Data type of return value</param>
      Function Parse(ByVal Expression As String ,Optional  ByVal DataType As String = "") As ParsedExpr
         Return New ParsedExpr(Parse__(uCalcHandle,Expression ,DataType))
      End Function

      ''' <summary> Returns the precedence level of an operator or the level between two operators</summary>
      Function Precedence(ByVal Operator1 As String, Optional  ByVal Operator2 As String = "") As  Int32
         Return Precedence_Overload_uCalc__(uCalcHandle,Operator1, Operator2)
      End Function

      ''' <summary> Combines properties into a numeric value that can be passed to ItemOf</summary>
      ''' <remarks><para>
      ''' ItemIsEnum properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
      ''' of the ItemOf function requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you
      ''' must pass one or more ItemIsEnum to Properties before passing it to ItemOf.
      ''' </para></remarks>
      Function Properties(ByVal p1 As ItemIsEnum, Optional  ByVal p2 As ItemIsEnum = ItemIsEnum.AnyItem, Optional  ByVal p3 As ItemIsEnum = ItemIsEnum.AnyItem, Optional  ByVal p4 As ItemIsEnum = ItemIsEnum.AnyItem) As  Int64
         Return Properties__(uCalcHandle,p1, p2, p3, p4)
      End Function

      ''' <summary> (Use Release instead)</summary>
      ''' <remarks><para>Use Release</para></remarks>
      Sub ReleaseuCalc()
         ReleaseuCalc__(uCalcHandle)
      End Sub

      ''' <summary> Removes any output decoration</summary>
      Sub ResetOutput()
         ResetOutput__(uCalcHandle)
      End Sub

      ''' <summary> Returns default settings</summary>
      ''' <remarks><para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
      ''' use Define(&quot;Default ~~ ...&quot;).</para></remarks>
      Function Default_ () As Item
         Return New Item(Default__(uCalcHandle))
      End Function

      ''' <summary> Allows a callback to decorate the output of EvalStr</summary>
      ''' <param name="FunctionAddress"> Address of callback function that formats the output</param>
      ''' <param name="Properties"> Name of datatype whose output will be formated and/or ranking among other output formats; leave blank to affect all data types and use default ranking.</param>
      Function SetOutput(ByVal FunctionAddress As uCalcDelegate , ByVal Properties As String) As Item
         Return New Item(SetOutput_Overload_uCalc_Callback__(uCalcHandle,FunctionAddress , Properties))
      End Function

      ''' <summary> Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
      ''' <param name="Definition"> Format definition for the output</param>
      ''' <remarks><para>
      ''' This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
      ''' of the following:
      ''' </para><para>
      ''' Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions
      ''' that evaluate to the specified data type.
      ''' </para><para>
      ''' InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed by
      ''' the other outputs in reverse order in which they were defined, ending with the output that was defined first.  You may
      ''' specify a different insertion point than the default with this parameter.
      ''' </para><para>
      ''' Expression:  This represents what the modified output should look like.  The expression consists of an identifier
      ''' (any temporary variable name), followed by an equal sign, followed by a string expression representing the value
      ''' that will be returned.  The variable holds the original value that would have been returned.
      ''' </para></remarks>
      Function SetOutput(ByVal Definition As String) As Item
         Return New Item(SetOutput_Overload_uCalc_NoCallback__(uCalcHandle,Definition))
      End Function

      ''' <summary> Returns the value located at the specified Pointer address</summary>
      Function ValueAt(ByVal ValuePtr As IntPtr, ByVal DataType As String) As String
         Return Marshal.PtrToStringAnsi(ValueAt__(uCalcHandle,ValuePtr, DataType)) 
      End Function

      ''' <summary> Returns the handle of the uCalc object</summary>
      ''' <remarks><para>Generally for internal use</para></remarks>
      Function Handle() As  IntPtr
         Return Handle_Overload_uCalc__(uCalcHandle)
      End Function

      ''' <summary> Releases a uCalc instance</summary>
      ''' <remarks><para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
      ''' associated with the uCalc instance.
      ''' </para><para>
      ''' Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
      ''' In C#, if there is only 1 uCalc object left, do not explicitely release it.
      ''' </para><para>
      ''' Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
      ''' automatically released from memory when the object goes out of scope.</para></remarks>
      Public Sub Release()
         ReleaseuCalc()
         uCalcHandle = Nothing 
      End Sub

      Public Sub Reset()
         ReleaseuCalc()
         uCalcHandle = CreateuCalc()
      End Sub

      Public Sub New()
         uCalcHandle = Nothing 
         uCalcHandle = CreateuCalc()
      End Sub

      Public Sub New(ByVal uCalcInstance As uCalc)
         uCalcHandle = uCalcInstance.uCalcHandle
      End Sub

      Public Sub New(ByVal InstHandle As IntPtr)
         uCalcHandle = InstHandle
      End Sub

      Public Sub New(ByVal LicenseType As LicenseTypeEnum, Optional  ByVal L1 As Int32 = 0, Optional  ByVal L2 As Int32 = 0, Optional  ByVal L3 As Int32 = 0, Optional  ByVal L4 As Int32 = 0, Optional  ByVal L5 As Int32 = 0)
         License(LicenseType, L1, L2, L3, L4, L5)
         uCalcHandle = CreateuCalc()
      End Sub

      Function PinAddr(ByVal Address As uCalcDelegate) As uCalcDelegate
         Static Addresses As New List(Of uCalcDelegate)
         Addresses.Add(Address)
         Return Address
      End Function
   End Class ' uCalc

#If Not SkipuCalcAutoDef Then
   Public uc As New uCalc
#End If
End Module
